# type: ignore
"""content

Revision ID: 3076beb33c1b
Revises: 
Create Date: 2021-01-08 23:09:26.868834

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "3076beb33c1b"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "medias",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("media_id", sa.Integer(), nullable=True),
        sa.Column("post_id", sa.Integer(), nullable=False),
        sa.Column("link", sa.String(), nullable=True),
        sa.Column("directory", sa.String(), nullable=True),
        sa.Column("filename", sa.String(), nullable=True),
        sa.Column("size", sa.Integer(), nullable=True),
        sa.Column("media_type", sa.String(), nullable=True),
        sa.Column("downloaded", sa.Integer(), nullable=True),
        sa.Column("created_at", sa.DATETIME(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("media_id"),
    )
    op.create_table(
        "stories",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("post_id", sa.Integer(), nullable=False),
        sa.Column("text", sa.String(), nullable=True),
        sa.Column("price", sa.Integer(), nullable=True),
        sa.Column("paid", sa.Integer(), nullable=True),
        sa.Column("created_at", sa.DATETIME(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("post_id"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("stories")
    op.drop_table("medias")
    # ### end Alembic commands ###
