# type: ignore

"""content

Revision ID: 0d4d92c0498e
Revises: b791cf213df9
Create Date: 2022-01-14 20:15:27.019051

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "0d4d92c0498e"
down_revision = "b791cf213df9"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("products", sa.Column("title", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("products", "title")
    # ### end Alembic commands ###
