# type: ignore
"""content

Revision ID: 29f675c35eee
Revises: 3076beb33c1b
Create Date: 2021-02-04 02:59:01.746229

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "29f675c35eee"
down_revision = "3076beb33c1b"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("medias", schema=None) as batch_op:
        batch_op.add_column(sa.Column("preview", sa.Integer(), nullable=True))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("medias", schema=None) as batch_op:
        batch_op.drop_column("preview")

    # ### end Alembic commands ###
