"""content

Revision ID: 37c4f2719d65
Revises: 0d4d92c0498e
Create Date: 2022-03-10 16:00:20.390009

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "37c4f2719d65"
down_revision = "0d4d92c0498e"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "profiles",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("user_id", sa.Integer(), nullable=False),
        sa.Column("username", sa.String(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("username"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("profiles")
    # ### end Alembic commands ###
